import typing

from babelfish import Country, CountryReverseConverter, CountryReverseError
from babelfish.converters import CaseInsensitiveDict


class GuessCountryConverter(CountryReverseConverter):
    def __init__(self, config: typing.Mapping[str, str]):
        self.synonyms = CaseInsensitiveDict(config)

    def convert(self, alpha2):
        return str(Country(alpha2))

    def reverse(self, name: str):
        try:
            return self.synonyms[name]
        except KeyError:
            pass

        if name.isupper() and len(name) == 2:
            try:
                return Country(name).alpha2
            except ValueError:
                pass

        for conv in (Country.fromname,):
            try:
                return conv(name).alpha2
            except CountryReverseError:
                pass

        raise CountryReverseError(name)



import typing

import pytest

from rebulk.match import Match

from trakit.words import blank_match, blank_release_names, to_combinations, to_sentence, to_words

from . import parameters_from_yaml


@pytest.mark.parametrize('value, expected', parameters_from_yaml(__file__, 'to_words'))
def test_to_words(value: str, expected: typing.List[typing.Mapping[str, typing.Union[str, int]]]):
    # given
    expected_matches = [Match(d['start'], d['end'], value=d['value'], input_string=value) for d in expected]

    # when
    actual = to_words(value)

    # then
    assert actual == expected_matches
    for w in actual:
        assert w.input_string[w.start:w.end] == value[w.start:w.end]
        assert w.value == value[w.start:w.end]


@pytest.mark.parametrize('value, expected', parameters_from_yaml(__file__, 'to_combinations'))
def test_to_combinations(value: str, expected: typing.List[typing.List[typing.Mapping[str, typing.Union[str, int]]]]):
    # given
    expected_combinations = [[
            Match(d['start'], d['end'], value=d['value'], input_string=value) for d in items
        ] for items in expected]

    # when
    actual = to_combinations(to_words(value), 4)

    # then
    assert actual == expected_combinations


@pytest.mark.parametrize('expected, data', parameters_from_yaml(__file__, 'to_sentence'))
def test_to_sentence(expected: str, data: typing.List[typing.Mapping[str, typing.Union[str, int]]]):
    # given
    combination = [Match(m['start'], m['end'], value=m['value']) for m in data]

    # when
    actual = to_sentence(combination)

    # then
    assert actual == expected


@pytest.mark.parametrize('value, data', parameters_from_yaml(__file__, 'blank_match'))
def test_blank_match(value: str, data: typing.Mapping[str, typing.Union[str, int]]):
    # given
    match = Match(data['start'], data['end'], input_string=value)
    expected = data['expected']

    # when
    actual = blank_match(match)

    # then
    assert actual == expected


@pytest.mark.parametrize('value, expected', parameters_from_yaml(__file__, 'blank_release_names'))
def test_blank_release_names(value: str, expected: str):
    # given

    # when
    actual = blank_release_names(value)

    # then
    assert actual == expected



import os
import typing

import yaml


def read_yaml(file_path):
    with open(file_path, 'r', encoding='utf-8') as f:
        return yaml.safe_load(f)


def parameters_from_yaml(test_filename: str, test_name: typing.Optional[str] = None):
    base_path = os.path.splitext(test_filename)[0]
    yml_file = f'{base_path}__{test_name}.yml' if test_name else f'{base_path}.yml'
    data = read_yaml(yml_file)

    for k, v in data.items():
        yield k, v



import json
import typing

try:
    from importlib.resources import files
except ImportError:
    from importlib_resources import files  # type: ignore[assignment,no-redef,import-not-found]


class Config:
    def __init__(self, config: typing.Optional[typing.Mapping[str, typing.Any]]):
        config_file = files(__package__).joinpath('data/config.json')
        with config_file.open('rb') as f:
            cfg: typing.Dict[str, typing.Any] = json.load(f)
        if config:
            cfg.update(config)

        self.ignored: typing.Set[str] = set(cfg.get('ignored', []))
        self.countries: typing.Mapping[str, str] = cfg.get('countries', {})
        self.languages: typing.Mapping[str, str] = cfg.get('languages', {})
        self.scripts: typing.Mapping[str, str] = cfg.get('scripts', {})
        self.regions: typing.Mapping[str, str] = cfg.get('regions', {})
        self.implicit_languages: typing.Mapping[str, str] = cfg.get('implicit-languages', {})



import typing

import babelfish


class Context(dict):
    def __init__(self, options: typing.Optional[typing.Mapping[str, typing.Any]] = None):
        super().__init__(options or {})
        language = self['expected_language'] if 'expected_language' in self else None
        if language and not isinstance(language, babelfish.Language):
            language = babelfish.Language.fromietf(str(language))
        self.expected_language: typing.Optional[babelfish.Language] = language
        self.type: typing.Literal['trackname', 'filename'] = self.get('type') or 'trackname'

    def accept(self, lang: babelfish.Language):
        if self.expected_language is None:
            return True
        if self.expected_language.alpha3 != lang.alpha3:
            return False
        if self.expected_language.script and self.expected_language != lang.script:
            return False

        return not self.expected_language.country or self.expected_language == lang.country



import typing

import pytest

from tests import parameters_from_yaml

from trakit import api


@pytest.mark.parametrize('value, expected', parameters_from_yaml(__file__, 'default_api'))
def test_default_api(value: str, expected: typing.Mapping[str, typing.Any]):
    # when
    actual = api.trakit(value)

    # then
    assert actual == expected


@pytest.mark.parametrize('value, expected', parameters_from_yaml(__file__, 'options'))
def test_options(value: str, expected: typing.Dict[str, typing.Any]):
    # given
    options = expected['options']
    del expected['options']

    # when
    actual = api.trakit(value, options)

    # then
    assert actual == expected



import json
import os
import re
import typing
from urllib.request import urlretrieve

from babelfish import Country, Language, LanguageReverseError

from unidecode import unidecode

CUR_PATH = os.path.dirname(__file__)
GENERATED_PATH = os.path.join(CUR_PATH, 'generated')
DOWNLOADED_PATH = os.path.join(CUR_PATH, 'downloaded')

suppress_re = re.compile(r"[\(\)'\,\.\u200f]")

with open(os.path.join(CUR_PATH, 'generator.json'), 'r', encoding='utf-8') as cfg_file:
    CONFIG: typing.Dict[str, typing.Any] = json.load(cfg_file)
    CONFIG['ignored_words'] = set(CONFIG['ignored_words'])
    CONFIG['countries']['ignore-extraction'] = set(CONFIG['countries']['ignore-extraction'])
    CONFIG['languages']['ignore-extraction'] = set(CONFIG['languages']['ignore-extraction'])
    additional_languages: typing.Mapping[str, str] = dict(CONFIG['languages']['additional'])
    for lang_syn, code in additional_languages.items():
        CONFIG['languages']['additional'][unidecode(lang_syn)] = code


class ConfigGenerator:
    def __init__(self,
                 country_demonyms: typing.Dict[str, str],
                 country_native_names: typing.Dict[str, typing.List[str]],
                 language_english_names: typing.Dict[str, str],
                 language_native_names: typing.Dict[str, str]):
        self.country_demonyms = country_demonyms
        self.country_native_names = country_native_names
        self.language_english_names = language_english_names
        self.language_native_names = language_native_names

    def register_duplicate(self,
                           name: str,
                           previous_value: str,
                           new_value: str,
                           duplicated: typing.Dict[str, typing.List[str]]):
        if name not in duplicated:
            duplicated[name] = []
        duplicated[name].append(previous_value)
        duplicated[name].append(new_value)

    def to_unique_values(self, values: typing.Iterable[str]):
        values = set([suppress_re.sub('', v).replace('-', ' ').strip() for v in values])
        unidecoded = [suppress_re.sub('', unidecode(v)) for v in values]
        values.update([v.strip() for v in unidecoded if '@' not in v])
        return [v for v in values if len(v) > 0]

    def register_country_synonym(self, country: Country, name: str,
                                 country_synonyms: typing.Dict[str, str],
                                 dup_country_syns: typing.Dict[str, typing.List[str]]):
        name = suppress_re.sub('', name)
        if name.upper() == country.name:
            return
        if name in country_synonyms and country_synonyms[name] != country.alpha2:
            self.register_duplicate(name, country_synonyms[name], country.alpha2, dup_country_syns)
            return

        country_synonyms[name] = country.alpha2

    @property
    def config(self):
        country_synonyms = dict(CONFIG['countries']['additional'])
        dup_country_syns = {}
        for alpha2, names in self.country_native_names.items():
            country = Country(alpha2)
            for name in self.to_unique_values(names):
                self.register_country_synonym(country, name, country_synonyms, dup_country_syns)

        for alpha2, demonym in self.country_demonyms.items():
            if alpha2 in CONFIG['countries']['ignored-ambiguous-demonyms']:
                continue

            country = Country(alpha2)
            for d in self.to_unique_values([demonym]):
                self.register_country_synonym(country, d, country_synonyms, dup_country_syns)

        for alpha2, demonyms in CONFIG['countries']['additional-demonyms'].items():
            country = Country(alpha2)
            for d in self.to_unique_values(demonyms):
                self.register_country_synonym(country, d, country_synonyms, dup_country_syns)

        language_synonyms = dict(CONFIG['languages']['additional'])
        ignore_extraction = CONFIG['countries']['ignore-extraction']
        dup_language_syns = {}
        all_names = dict(self.language_english_names)
        all_names.update(self.language_native_names)
        country_re = re.compile(r'^(?P<pre>.*)[\(\uff08](?P<country>[^)\uff09]+)[\)\uff09](?P<pos>.*)$')
        for ietf, synonym in all_names.items():
            language = Language.fromietf(ietf)
            m = country_re.match(synonym)
            country_extracted = m and language.country and language.alpha3 not in ignore_extraction
            c = m.group('country').strip() if country_extracted else None
            syn = (m.group('pre') or m.group('pos')).strip() if country_extracted else synonym
            if c:
                for name in self.to_unique_values([c]):
                    self.register_country_synonym(language.country, name, country_synonyms, dup_country_syns)
            lang_code = str(language) if language.script else language.alpha3
            for name in self.to_unique_values([syn]):
                name = suppress_re.sub('', name)
                if name.lower() == language.name.lower():
                    continue
                if name in language_synonyms and language_synonyms[name] != lang_code:
                    self.register_duplicate(name, language_synonyms[name], lang_code, dup_language_syns)
                    continue

                language_synonyms[name] = lang_code

        if dup_country_syns:
            print(f'Duplicated country synonyms: {dup_country_syns}')
        if dup_language_syns:
            print(f'Duplicated language synonyms: {dup_language_syns}')

        scripts = dict(CONFIG['scripts'])
        scripts.update({unidecode(k): v for k, v in scripts.items()})

        regions = dict(CONFIG['regions'])
        regions.update({unidecode(k): v for k, v in regions.items()})

        return {
            'ignored': sorted([v for v in CONFIG['ignored_words']]),
            'countries': country_synonyms,
            'languages': language_synonyms,
            'implicit-languages': CONFIG['languages']['implicit'],
            'scripts': scripts,
            'regions': regions
        }

    def generate(self):
        config = self.config
        with open(os.path.join(GENERATED_PATH, 'config.json'), 'w', encoding='utf-8') as out:
            json.dump(config, out, ensure_ascii=False, indent=2, sort_keys=True)


class Generator:

    def __init__(self):
        self.country_demonyms: typing.Dict[str, str] = {}
        self.country_additional_demonyms: typing.Dict[str, typing.List[str]] = {}
        self.country_native_names: typing.Dict[str, typing.List[str]] = {}
        self.language_english_names: typing.Dict[str, str] = {}
        self.language_native_names: typing.Dict[str, str] = {}

    def generate(self):
        self.download_files()
        self.process_countries()
        self.process_languages()
        self.generate_files()
        ConfigGenerator(self.country_demonyms,
                        self.country_native_names,
                        self.language_english_names,
                        self.language_native_names).generate()

    def generate_files(self):
        if not os.path.isdir(GENERATED_PATH):
            os.mkdir(GENERATED_PATH)
        with open(os.path.join(GENERATED_PATH, 'country-demonyms.json'), 'w', encoding='utf-8') as out:
            json.dump(self.country_demonyms, out, ensure_ascii=False, indent=2)
        with open(os.path.join(GENERATED_PATH, 'country-native-names.json'), 'w', encoding='utf-8') as out:
            json.dump(self.country_native_names, out, ensure_ascii=False, indent=2)
        with open(os.path.join(GENERATED_PATH, 'language-english-names.json'), 'w', encoding='utf-8') as out:
            json.dump(self.language_english_names, out, ensure_ascii=False, indent=2)
        with open(os.path.join(GENERATED_PATH, 'language-native-names.json'), 'w', encoding='utf-8') as out:
            json.dump(self.language_native_names, out, ensure_ascii=False, indent=2)
        with open(os.path.join(GENERATED_PATH, 'country-demonyms.txt'), 'w', encoding='utf-8') as out:
            for k, v in self.country_demonyms.items():
                if v:
                    if k in CONFIG['countries']['ignored-ambiguous-demonyms']:
                        out.write('#')
                    out.write(f'{k}={v}\n')

    def download_files(self):
        countries_path = os.path.join(DOWNLOADED_PATH, 'countries.json')
        language_mapping_list_path = os.path.join(DOWNLOADED_PATH, 'language-mapping-list.js')
        language_mapping_json_path = os.path.join(DOWNLOADED_PATH, 'language-mapping-list.json')

        if not os.path.isdir(DOWNLOADED_PATH):
            os.mkdir(DOWNLOADED_PATH)
        if not os.path.isfile(countries_path):
            urlretrieve(CONFIG['urls']['countries'], countries_path)
        if not os.path.isfile(language_mapping_list_path):
            urlretrieve(CONFIG['urls']['local-language-names'], language_mapping_list_path)
        if not os.path.isfile(language_mapping_json_path):
            with open(language_mapping_list_path, 'r', encoding='utf-8') as f:
                content: str = f.read()
                content = content[content.index('  return {')+9:content.index('  };')+3]
                content = re.sub(r"'(?P<ietf>\w+([\-\@]\w+)?)'\:", r'"\g<ietf>":', content)
                content = re.sub(r"\b(?P<name>\w+)\b\:", r'"\g<name>":', content)
                with open(language_mapping_json_path, 'w', encoding='utf-8') as out:
                    out.write(content)

    def process_country_demonym(self,
                                country: Country,
                                c: typing.Mapping[str, typing.Any]):
        demonym = c['demonyms']['eng']['m']
        if demonym:
            self.country_demonyms[country.alpha2] = demonym

    def process_country_native_names(self,
                                     country: Country,
                                     c: typing.Mapping[str, typing.Any]):
        for v in c['name']['native'].values():
            name = v['common']
            if country.alpha2 not in self.country_native_names:
                self.country_native_names[country.alpha2] = []
            names = self.country_native_names[country.alpha2]
            if name not in names:
                names.append(name)

    def process_language_names(self,
                               language: Language,
                               names: typing.Mapping[str, str]):
        ietf = str(language)
        self.language_english_names[ietf] = names['englishName']
        self.language_native_names[ietf] = names['nativeName']

    def process_countries(self):
        with open(os.path.join(DOWNLOADED_PATH, 'countries.json'), 'r', encoding='utf-8') as f:
            countries = json.load(f)
            for c in countries:
                try:
                    country = Country(c['cca2'])
                    self.process_country_demonym(country, c)
                    self.process_country_native_names(country, c)
                except ValueError:
                    print(f'Unknown babelfish.Country {c["cca2"]}: {c["name"]["common"]}')
        self.country_demonyms.update(CONFIG['countries']['overriden-demonyms'])

    def process_languages(self):
        with open(os.path.join(DOWNLOADED_PATH, 'language-mapping-list.json'), 'r', encoding='utf-8') as f:
            languages = json.load(f)
            for ietf, names in languages.items():
                if ietf in CONFIG['languages']['ignore-extraction']:
                    continue

                try:
                    language = Language.fromietf(ietf)
                    self.process_language_names(language, names)
                except (ValueError, LanguageReverseError):
                    print(f'Unknown babelfish.Language {ietf}: {names}')



import argparse
import json
import logging
import sys
import typing

import babelfish

from trakit import TrakItApi, __version__

logging.basicConfig(stream=sys.stdout, format='%(message)s')
logging.getLogger('CONSOLE').setLevel(logging.INFO)
logging.getLogger('trakit').setLevel(logging.WARNING)

console = logging.getLogger('CONSOLE')
logger = logging.getLogger('trakit')


def build_argument_parser() -> argparse.ArgumentParser:
    """Build the argument parser."""
    opts = argparse.ArgumentParser()
    opts.add_argument(
        dest='value',
        help='track title to guess',
        type=str,
    )

    conf_opts = opts.add_argument_group('Configuration')
    conf_opts.add_argument(
        '-l',
        '--expected-language',
        dest='expected_language',
        help='The expected language to be guessed',
        type=str,
    )
    conf_opts.add_argument(
        '-t',
        '--type',
        dest='type',
        help='The input type: trackname or filename. Default is trackname',
        type=str,
    )

    output_opts = opts.add_argument_group('Output')
    output_opts.add_argument(
        '--debug',
        action='store_true',
        dest='debug',
        help='Print information for debugging trakit and for reporting bugs.'
    )
    output_opts.add_argument(
        '-y',
        '--yaml',
        action='store_true',
        dest='yaml',
        help='Display output in yaml format'
    )

    information_opts = opts.add_argument_group('Information')
    information_opts.add_argument('--version', action='version', version=__version__)

    return opts


def _as_yaml(value: str, info: typing.Mapping[str, typing.Any]) -> str:
    """Convert info to string using YAML format."""
    import yaml

    def default_representer(r: yaml.representer.SafeRepresenter, data: typing.Any):
        return r.represent_scalar('tag:yaml.org,2002:str', str(data))

    yaml.representer.SafeRepresenter.add_representer(babelfish.Language, default_representer)

    return yaml.safe_dump({value: dict(info)}, allow_unicode=True, sort_keys=False)


def _as_json(info: typing.Mapping[str, typing.Any]) -> str:
    """Convert info to string using JSON format."""
    return json.dumps(info, ensure_ascii=False, indent=2, default=str)


def dump(value: str, info: typing.Mapping[str, typing.Any], opts: argparse.Namespace) -> str:
    """Convert info to string using json or yaml format."""
    if opts.yaml:
        return _as_yaml(value, info)

    return _as_json(info)


def trakit(value: str, opts: argparse.Namespace) -> typing.Mapping:
    """Extract video metadata."""
    if not opts.yaml:
        console.info('Parsing: %s', value)
    options = {k: v for k, v in vars(opts).items() if v is not None}
    info = TrakItApi().trakit(value, options)
    console.info('TrakIt %s found: ', __version__)
    console.info(dump(value, info, opts))
    return info


def execute(args: typing.Optional[typing.List[str]] = None):
    """Execute main function for entry point."""
    argument_parser = build_argument_parser()
    args = args or sys.argv[1:]
    opts = argument_parser.parse_args(args)

    if opts.debug:
        logger.setLevel(logging.DEBUG)
        logging.getLogger('rebulk').setLevel(logging.DEBUG)

    return trakit(opts.value, opts)


def main(args: typing.Optional[typing.List[str]] = None):
    execute(args)


if __name__ == '__main__':
    main(sys.argv[1:])



import typing

from babelfish import (
    COUNTRIES,
    Country,
    CountryReverseError,
    LANGUAGE_MATRIX,
    Language,
    LanguageReverseError,
    SCRIPTS,
    Script,
    country_converters,
    language_converters
)
from babelfish.converters import CaseInsensitiveDict

from rebulk import Rebulk
from rebulk.match import Match

from trakit.config import Config
from trakit.context import Context
from trakit.converters.country import GuessCountryConverter
from trakit.converters.language import GuessLanguageConverter
from trakit.words import (
    blank_base_name_and_extension,
    blank_match,
    blank_release_names,
    to_combinations,
    to_match,
    to_sentence,
    to_words)


def _convert(value: str, context: Context):
    if context.type == 'filename':
        for conv in (Language.fromietf,
                     Language.fromalpha3b,
                     Language.fromalpha2):
            try:
                return conv(value)
            except (ValueError, LanguageReverseError):
                continue

    try:
        return Language.fromguess(value)
    except LanguageReverseError:
        return None


class LanguageFinder:

    def __init__(self, config: Config):
        self.country_max_words = 1
        for k, v in COUNTRIES.items():
            self.country_max_words = max(self.country_max_words, v.count(' '))

        self.language_max_words = 1
        for v in LANGUAGE_MATRIX:
            self.language_max_words = max(self.language_max_words, v.name.count(' '))

        self.script_max_words = 1
        for v in config.scripts.keys():
            self.script_max_words = max(self.script_max_words, v.count(' '))

        self.region_max_words = 1
        for v in config.regions.keys():
            self.region_max_words = max(self.region_max_words, v.count(' '))

        SCRIPTS['419'] = 'Latin America and the Caribbean'  # Until babelfish support UN.M49
        country_converters['guess'] = GuessCountryConverter(config.countries)
        language_converters['guess'] = GuessLanguageConverter(config.languages)
        self.regions = CaseInsensitiveDict(config.regions)
        self.scripts = CaseInsensitiveDict(config.scripts)
        self.common_words = CaseInsensitiveDict(dict.fromkeys(config.ignored, 0))
        self.implicit = CaseInsensitiveDict(config.implicit_languages)

    def _find_country(self, value: str):
        combinations = to_combinations(to_words(value), self.country_max_words)
        for c in combinations:
            code = to_sentence(c)
            try:
                return to_match(c, Country.fromguess(code))
            except CountryReverseError:
                continue

    def _find_script(self, value: str):
        combinations = to_combinations(to_words(value), self.script_max_words)
        for c in combinations:
            code = to_sentence(c)
            try:
                return to_match(c, Script(self.scripts.get(code, code)))
            except ValueError:
                continue

    def _find_region(self, value: str):
        combinations = to_combinations(to_words(value), self.region_max_words)
        for c in combinations:
            code = to_sentence(c)
            try:
                return to_match(c, Script(self.regions.get(code, code)))
            except ValueError:
                continue

    def _find_implicit_language(self, combinations: typing.List[typing.List[Match]]):
        for c in combinations:
            sentence = to_sentence(c)
            if sentence in self.implicit:
                return to_match(c, Language.fromietf(self.implicit[sentence]))

            region = self._find_region(sentence)
            if region and region.value.code in self.implicit:
                lang = Language.fromietf(self.implicit[region.value.code])
                return Match(region.start, region.end, value=lang, input_string=region.input_string)

            try:
                country = Country.fromguess(sentence)
                if country.alpha2 in self.implicit:
                    lang = Language.fromietf(self.implicit[country.alpha2])
                    if lang.name.lower() == sentence.lower():
                        lang = Language.fromname(sentence)

                    return to_match(c, lang)
            except CountryReverseError:
                pass

    def accept_word(self, string: str):
        return string.lower() not in self.common_words and not string.isnumeric()

    def find_language(self, value: str, context: Context):
        value = blank_base_name_and_extension(value) if context.type == 'filename' else blank_release_names(value)
        all_words = to_words(value, predicate=self.accept_word)
        combinations = to_combinations(all_words, self.language_max_words)
        implicit_lang = self._find_implicit_language(combinations)
        implicit_accepted = implicit_lang and context.accept(implicit_lang.value)

        if implicit_accepted and implicit_lang.value.script and implicit_lang.value.script.code.isnumeric():
            return implicit_lang
        elif implicit_lang and not implicit_accepted:
            value = blank_match(implicit_lang)
            all_words = to_words(value, predicate=self.accept_word)
            combinations = to_combinations(all_words, self.language_max_words)

        for c in combinations:
            language_sentence = to_sentence(c)
            lang: typing.Optional[Language] = _convert(language_sentence, context)
            if lang is None:
                continue

            match_lang = to_match(c, lang)
            remaining_sentence = blank_match(match_lang)
            for combination in to_combinations(to_words(remaining_sentence), self.country_max_words):
                sentence = to_sentence(combination)
                country = self._find_country(sentence)
                if country:
                    try:
                        # discard country if value is actually the language name
                        Language.fromguess(country.raw)
                    except LanguageReverseError:
                        lang = Language(lang.alpha3, country=country.value, script=lang.script)
                    break

                region = self._find_region(sentence)
                if region:
                    lang = Language(lang.alpha3, country=lang.country, script=region.value)
                    break

                script = self._find_script(sentence)
                if script:
                    lang = Language(lang.alpha3, country=lang.country, script=script.value)
                    break

            if implicit_accepted and implicit_lang.value.alpha3 == lang.alpha3 and not lang.country and not lang.script:
                return implicit_lang

            if context.accept(lang):
                return to_match(c, lang)

        if implicit_accepted:
            return implicit_lang

    def find(self, value: str, context: Context):
        match = self.find_language(value, context)
        if match:
            return match.start, match.end, {'value': match.value}


def language(config: Config):
    rebulk = Rebulk()
    rebulk.functional(LanguageFinder(config).find, name='language')

    return rebulk



from importlib import metadata

__title__ = metadata.metadata(__package__)['name']
__version__ = metadata.version(__package__)
__short_version__ = '.'.join(__version__.split('.')[:2])
__author__ = metadata.metadata(__package__)['author']
__license__ = metadata.metadata(__package__)['license']

del metadata

from .api import TrakItApi, trakit



import re
from functools import partial

from rebulk import Rebulk
from rebulk.validators import chars_surround

from trakit.config import Config
from trakit.language import language
from trakit.words import seps


def configure(config: Config):
    seps_surround = partial(chars_surround, seps)

    others = Rebulk()
    others.defaults(ignore_case=True, validator=seps_surround)
    others.regex_defaults(flags=re.IGNORECASE,
                          abbreviations=[(r'-', rf'[{re.escape("".join(seps))}]')],
                          validator=seps_surround)
    for name in ('forced', 'commentary', 'external'):
        others.string(name, name=name, value=True)

    others.string('sdh', name='hearing_impaired', value=True)
    others.string('alternate', name='version', value='alternate')
    others.string('descriptive', name='descriptive', value=True)
    others.regex('cc', 'closed-captions?', name='closed_caption', value=True)

    rebulk = Rebulk()
    rebulk.rebulk(language(config))
    rebulk.rebulk(others)

    return rebulk



import typing

import pytest

from tests import parameters_from_yaml

from trakit.__main__ import execute


@pytest.mark.parametrize('name, data', parameters_from_yaml(__file__, 'main'))
def test_main(name: str, data: typing.Mapping[str, typing.Any]):
    # given
    args = data['args']
    expected = data['expected']

    # when
    actual = execute(args)

    # then
    assert actual == expected



import typing

from babelfish import Language, LanguageReverseConverter, LanguageReverseError
from babelfish.converters import CaseInsensitiveDict


class GuessLanguageConverter(LanguageReverseConverter):
    def __init__(self, config: typing.Mapping[str, str]):
        self.synonyms = CaseInsensitiveDict()
        for synonym, code in config.items():
            lang = Language.fromietf(code) if '-' in code else Language(code)
            self.synonyms[synonym] = (lang.alpha3, lang.country.alpha2 if lang.country else None, lang.script)

    def convert(self, alpha3: str, country=None, script=None):
        return str(Language(alpha3, country, script))

    def reverse(self, name: str):
        try:
            return self.synonyms[name]
        except KeyError:
            pass

        for conv in (Language.fromname,):
            try:
                reverse = conv(name)
                return reverse.alpha3, reverse.country, reverse.script
            except (ValueError, LanguageReverseError):
                pass

        raise LanguageReverseError(name)



import json
import os

import trakit.config

from .generator import GENERATED_PATH, Generator

TRACKIT_PATH = os.path.join(os.path.dirname(trakit.__file__), 'data')


def test_generate_config():
    # given
    with open(os.path.join(TRACKIT_PATH, 'config.json'), 'r', encoding='utf-8') as f:
        expected = json.load(f)
    Generator().generate()

    # when
    with open(os.path.join(GENERATED_PATH, 'config.json'), 'r', encoding='utf-8') as f:
        actual = json.load(f)

    # then
    assert expected == actual



import typing

from trakit.config import Config
from trakit.context import Context
from trakit.patterns import configure


class TrakItApi:

    def __init__(self, config: typing.Optional[typing.Mapping[str, typing.Any]] = None):
        self.rebulk = configure(Config(config))

    def trakit(self, string: str, options: typing.Optional[typing.Mapping[str, typing.Any]] = None):
        """Return a mapping of extracted information."""
        matches = self.rebulk.matches(string, Context(options))
        guess: typing.Mapping[str, typing.Any] = matches.to_dict()
        return guess


default_api = TrakItApi()


def trakit(string: str, options: typing.Optional[typing.Mapping[str, typing.Any]] = None):
    return default_api.trakit(string, options)



import re
import typing
from pathlib import Path

from rebulk.match import Match

seps = frozenset(r' [](){}+*|=-_~#/\\.,;:' + '\uff08\uff09')
suppress_chars = frozenset("'")
release_name_re = re.compile(r'(?P<release>[^\.\s]+(?:\.[^\.\s]+){2,})')


def to_words(value: str,
             separators: typing.FrozenSet[str] = seps,
             ignore_chars: typing.FrozenSet[str] = suppress_chars,
             predicate: typing.Callable[[str], bool] = lambda x: True):
    input_string = value
    start = 0
    i = 0
    word = ''
    words: typing.List[Match] = []
    for c in input_string:
        i += 1
        if c in ignore_chars:
            continue

        if c not in separators:
            word += c
            continue

        if not word:
            start = i
            continue

        end = i - 1
        if not predicate(value[start:end]):
            input_string = blank(input_string, start, end)
        else:
            words.append(Match(start, i - 1, value=word))

        word = ''
        start = i

    if word:
        if not predicate(value[start:]):
            input_string = blank(input_string, start, len(input_string))
        else:
            words.append(Match(start, i, value=word))

    for w in words:
        w.input_string = input_string

    return words


def to_combinations(words: typing.List[Match], max_items: int):
    results: typing.List[typing.List[Match]] = []
    n_words = len(words)
    cur_size = min(max_items, n_words)
    start = 0
    while cur_size > 0:
        end = start + cur_size
        if end > n_words:
            start = 0
            cur_size -= 1
            continue

        results.append(words[start:end])
        start += 1

    return results


def to_sentence(combination: typing.List[Match]):
    return ' '.join([c.value for c in combination])


def to_match(combination: typing.List[Match], value: typing.Any):
    start = combination[0].start
    end = combination[-1].end
    input_string = combination[0].input_string

    return Match(start, end, value=value, input_string=input_string)


def blank(string: str, start: int, end: int):
    return string[:start] + ''.ljust(end - start, ' ') + string[end:]


def blank_match(match: Match):
    assert match.input_string is not None
    return blank(match.input_string, match.start, match.end)


def blank_release_names(value: str):
    result = value
    match = release_name_re.search(value)
    while match:
        result = blank(result, match.start('release'), match.end('release'))
        match = release_name_re.search(value, match.end('release'))

    return result


def blank_base_name_and_extension(value: str):
    p = Path(value)
    stems = p.stem.split('.')

    return '.'.join(stems[1:])



import pytest

from trakit import trakit

from . import parameters_from_yaml


@pytest.mark.parametrize('value, expected', parameters_from_yaml(__file__))
def test_scenarios(value, expected):
    # given

    # when
    actual = trakit(value)

    # then
    assert dict(actual) == expected
